#include <mega48.h>
#include <stdio.h> 
#include <stdlib.h>
#include <STRING.H> 
#include <ctype.H>
#include <i2c.h> 
#include <delay.h>
#include <math.h>    

#include <functions.c>
// Yak module
#define TWINT       0b10000000
#define TWEA        0b01000000
#define TWSTA       0b00100000
#define TWSTO       0b00010000
#define TWWC        0b00001000
#define TWEN        0b00000100
#define TWIE        0b00000001

#define SLA_W       0b10100000
#define SLA_R       0b10100001

#define EEPROM_BUS_ADDRESS 0xa0

unsigned char ext_eeprom_read(unsigned int address) 
{
        unsigned char data, addrh, addrl;
        addrh=(((address<<4)>>12)<<1);
        addrl=(char)address;
        i2c_start();
        i2c_write(EEPROM_BUS_ADDRESS | addrh);
        i2c_write(addrl);
        i2c_start();
        i2c_write(EEPROM_BUS_ADDRESS | 1 | addrh);
        data=i2c_read(1);
        i2c_stop();
        return data;
}

void ext_eeprom_write(unsigned char address, unsigned char data) 
{
        i2c_start();
        i2c_write(EEPROM_BUS_ADDRESS);
        i2c_write(address);
        i2c_write(data);
        i2c_stop();
        delay_ms(10);
}

// ================= Write byte to internal EEPROM ===========
void eeprom_write(unsigned int uiAddress, unsigned char ucData)
{
        #asm("cli")
        while(EECR.1);
        EEAR=uiAddress;
        EEDR=ucData;
        EECR.2=1;
        EECR.1=1;
        #asm("sei")
} 

// ================= Read byte from internal EEPROM =========
unsigned int eeprom_read(unsigned int uiAddress)
{       
        #asm("cli")
        while(EECR.1);
        EEAR=uiAddress;
        EECR.0=1;
        return EEDR;
        #asm("sei")
} 

// ================= HEX to DEC conversion ===================
unsigned int hexdec(char* str)
{       
        signed long i,j,a,b,c;
        unsigned int ln=0, tmpint=0;
        ln=strlen(str)-1;
        //printf(" \r\n\r\n unsigned int hexdec(char* str) \r\n - str = %s \r\n ",str);
        //printf(" \r\n - ln = %X \r\n ",ln);
        
        i=ln; j=0;
        while (i>=0)
        { 
                if (i==0) a=1;
                if (i==1) a=0x0F;
                if (i==2) a=0xFF;
                if (i==3) a=0xFFF;
                if (i==4) a=0xFFFF; 
                
                b=toint(str[j]); c=a*b;
                tmpint=tmpint+c;
                printf(" \r\n a = %u ; b = %u ; c = %u ; \r\n ",a,b,c);
                i--; j++;
        }
        
        printf(" \r\n - tmpint = %X \r\n ",tmpint);
        return tmpint;
}

#asm
   .equ __i2c_port=0x0B ;PORTD
   .equ __sda_bit=5
   .equ __scl_bit=6
#endasm
 
#define RXB8 1
#define TXB8 0
#define UPE 2
#define OVR 3
#define FE 4
#define UDRE 5
#define RXC 7

#define FRAMING_ERROR (1<<FE)
#define PARITY_ERROR (1<<UPE)
#define DATA_OVERRUN (1<<OVR)
#define DATA_REGISTER_EMPTY (1<<UDRE)
#define RX_COMPLETE (1<<RXC)

// USART Receiver buffer
#define RX_BUFFER_SIZE0 32
char rx_buffer0[RX_BUFFER_SIZE0];
unsigned char rx_wr_index0,rx_counter0;//,rx_rd_index0;
bit rx_buffer_overflow0;

//AT PROTOCOL
bit ATECHO=1;
#define MAX_INT_EEPROM 0xFF
#define MAX_EXT_EEPROM 0x3FFF

flash char AT_C[]="AT";
flash char AT_E0[]="ATE0";
flash char AT_E1[]="ATE1";
flash char AT_INFO[]="AT+INFO";
flash char AT_IEEPR[]="AT+IEEPR=";
flash char AT_IEEPW[]="AT+IEEPW=";
flash char AT_EEEPR[]="AT+EEEPR=";
flash char AT_EEEPW[]="AT+EEEPW=";

#pragma warn-

// USART Receiver interrupt service routine
interrupt [USART_RXC] void usart_rx_isr(void)
{
signed char scmptmp=-1;
char i=0, j=0;
char attmp[16];
unsigned int tmpint=0,EEP=0;

//unsigned char data;

char status,data;
status=UCSR0A;
data=UDR0;
if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)
   {
        rx_buffer0[rx_wr_index0]=data;
        if (++rx_wr_index0 == RX_BUFFER_SIZE0)
        {
                rx_wr_index0=0;
                rx_counter0=0;
                rx_wr_index0=0;
                tmpint=0;
                for (i=0; i==RX_BUFFER_SIZE0; i++) rx_buffer0[i]=0;
                for (i=0; i==16; i++) attmp[i]=0;
        };
        if (++rx_counter0 == RX_BUFFER_SIZE0)
        {
                rx_counter0=0;
                rx_buffer_overflow0=1;
                rx_counter0=0;
                rx_wr_index0=0;
                tmpint=0;
                for (i=0; i==RX_BUFFER_SIZE0; i++) rx_buffer0[i]=0;
                for (i=0; i==16; i++) attmp[i]=0;
        };
          
        if (data==13)
        {
                rx_buffer0[rx_wr_index0-1]=NULL;
                
                //==============AT COMMAND SECTOR BEGIN=================
                scmptmp=strcmpf(rx_buffer0,AT_C);
                if (scmptmp==0)
                {
                        if (ATECHO==1) printf(" \r\n\r\n%s ",rx_buffer0);
                        printf(" \r\n\r\nOK\r\n ",rx_buffer0);
                        goto GT;    
                };
 /*               
                //------AT command echo off-------------
                scmptmp=strcmpf(rx_buffer0,AT_E0);
                if (scmptmp==0)
                {
                        if (ATECHO==1) printf(" \r\n\r\n%s ",rx_buffer0);
                        printf(" \r\nCOMMAND ECHO OFF\r\n\r\nOK\r\n ");
                        ATECHO=0;
                        goto GT;    
                };
                
                //------AT command echo on--------------
                scmptmp=strcmpf(rx_buffer0,AT_E1);
                if (scmptmp==0)
                {
                        if (ATECHO==1) printf(" \r\n\r\n%s ",rx_buffer0);
                        printf(" \r\nCOMMAND ECHO ON\r\n\r\nOK\r\n ");
                        ATECHO=1;
                        goto GT;    
                };
                                
                //------Show info-----------------------
                scmptmp=strcmpf(rx_buffer0,AT_INFO);
                if (scmptmp==0)
                {
                        if (ATECHO==1) printf(" \r\n\r\n%s ",rx_buffer0);
                        printf("\r\n ATMEL ATMEGA48 \r\n Max.EEPROM address = 0x%X \r\n Autor: Yak\r\n Date: 07.10.2006\r\n\r\n OK\r\n ",MAX_INT_EEPROM);
                        goto GT;    
                };
 */               
                //-------Internal EEPROM read-----------
                scmptmp=strncmpf(rx_buffer0,AT_IEEPR,9);
                if (scmptmp==0)
                {       
                        while (rx_buffer0[i+9]!=NULL)
                        {
                                attmp[i]=rx_buffer0[i+9];
                                if (!( (isdigit(attmp[i])) || (isxdigit(attmp[i])) )) goto FD;
                                i++;
                        };
                        tmpint=hexdec(attmp);
                        
                        if ((tmpint<0)||(tmpint>MAX_INT_EEPROM))
                        {
FD:
                                if (ATECHO==1) printf("\r\n\r\n%s",rx_buffer0);
                                printf("\r\n\r\nVALUE ERROR\r\n");
                                goto GT; 
                        };
                        
                        EEP=eeprom_read(tmpint);
                        if (ATECHO==1) printf(" \r\n\r\n%s ",rx_buffer0);
                        printf(" \r\n+IEEPR=0x%X\r\n\r\nOK\r\n ",EEP);
                        goto GT;    
                };
                
                //-------Internal EEPROM write----------
                scmptmp=strncmpf(rx_buffer0,AT_IEEPW,9);
                if (scmptmp==0)
                { 
                        while (rx_buffer0[i+9]!=',')
                        {
                                attmp[i]=rx_buffer0[i+9];
                                if (!( (isdigit(attmp[i])) || (isxdigit(attmp[i])) )) goto GD;
                                i++;
                        };
                        tmpint=hexdec(attmp);
                        
                        if ((tmpint<0)||(tmpint>MAX_INT_EEPROM))
                        {
GD:
                                if (ATECHO==1) printf("\r\n\r\n%s",rx_buffer0);
                                printf("\r\n\r\nADDRESS ERROR\r\n");
                                goto GT; 
                        };
                        
                        for (j=0; j==16; j++) attmp[j]=0;
                        j=0;

                        while (rx_buffer0[i+10]!=NULL)
                        {
                                attmp[j]=rx_buffer0[i+10];
                                if (!( (isdigit(attmp[j])) || (isxdigit(attmp[j])) )) goto GD;
                                i++;
                                j++;
                        };
                        EEP=hexdec(attmp);

                        eeprom_write(tmpint,EEP);
                        EEP=eeprom_read(tmpint);
                        if (ATECHO==1) printf("\r\n\r\n%s",rx_buffer0);
                        printf(" \r\n\r\n ADDRESS [0x%X] \r\n VALUE [0x%X] \r\n WRITE OK\r\n\r\n  ",tmpint,EEP);
                        goto GT;
                
                };
                
                
                //-------External EEPROM read-----------
                scmptmp=strncmpf(rx_buffer0,AT_EEEPR,9);
                if (scmptmp==0)
                {       
                        while (rx_buffer0[i+9]!=NULL)
                        {
                                attmp[i]=rx_buffer0[i+9];
                                if (!( (isdigit(attmp[i])) || (isxdigit(attmp[i])) )) goto FX;
                                i++;
                        };

                        tmpint=hexdec(attmp);  
                        
                        //printf(" \r\n \r\n tmpint=hexdec(attmp); \r\n - attmp = %s \r\n ",attmp);
                        //printf(" \r\n - tmpint = %X \r\n ",tmpint);
                        
                        if ((tmpint<0)||(tmpint>MAX_EXT_EEPROM))
                        {
FX:
                                if (ATECHO==1) printf("\r\n\r\n%s",rx_buffer0);
                                printf("\r\n\r\nVALUE ERROR\r\n");
                                goto GT; 
                        };
                        
                        EEP=ext_eeprom_read(tmpint);
                        if (ATECHO==1) printf(" \r\n\r\n%s ",rx_buffer0);
                        printf(" \r\n+EEEPR=0x%X\r\n\r\nOK\r\n ",EEP);
                        goto GT;    
                };
                
                //-------External EEPROM write----------
                scmptmp=strncmpf(rx_buffer0,AT_EEEPW,9);
                if (scmptmp==0)
                { 
                        while (rx_buffer0[i+9]!=',')
                        {
                                attmp[i]=rx_buffer0[i+9];
                                if (!( (isdigit(attmp[i])) || (isxdigit(attmp[i])) )) goto GW;
                                i++;
                        };
                        tmpint=hexdec(attmp);
                        
                        if ((tmpint<0)||(tmpint>MAX_EXT_EEPROM))
                        {
GW:
                                if (ATECHO==1) printf("\r\n\r\n%s",rx_buffer0);
                                printf("\r\n\r\nADDRESS ERROR\r\n");
                                goto GT; 
                        };
                        
                        for (j=0; j==16; j++) attmp[j]=0;
                        j=0;

                        while (rx_buffer0[i+10]!=NULL)
                        {
                                attmp[j]=rx_buffer0[i+10];
                                if (!( (isdigit(attmp[j])) || (isxdigit(attmp[j])) )) goto GD;
                                i++;
                                j++;
                        };
                        EEP=hexdec(attmp);

                        ext_eeprom_write(tmpint,EEP);
                        EEP=ext_eeprom_read(tmpint);
                        if (ATECHO==1) printf("\r\n\r\n%s",rx_buffer0);
                        printf(" \r\n\r\n ADDRESS [0x%X] \r\n VALUE [0x%X] \r\n WRITE OK\r\n\r\n  ",tmpint,EEP);
                        goto GT;
                
                };
                
                printf(" \r\n\r\n%s\r\n\r\nERROR\r\n ",rx_buffer0);

                //-------------AT COMMAND SECTOR END-----------------
GT:
                rx_counter0=0;
                rx_wr_index0=0;
                tmpint=0;
                for (i=0; i==RX_BUFFER_SIZE0; i++) rx_buffer0[i]=0;
                for (i=0; i==16; i++) attmp[i]=0;
        };
        
        printf("%c",data);
   };
}

#pragma warn+

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
// Place your code here

}

// Declare your global variables here

void main(void)
{
// Declare your local variables here

// Crystal Oscillator division factor: 1
#pragma optsize-
CLKPR=0x80;
CLKPR=0x00;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

// Input/Output Ports initialization
// Port B initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=P State6=P State5=P State4=P State3=P State2=P State1=P State0=P 
PORTB=0xFF;
DDRB=0x00;

// Port C initialization
// Func6=In Func5=In Func4=In Func3=In Func2=Out Func1=In Func0=In 
// State6=P State5=P State4=P State3=P State2=0 State1=P State0=P 
PORTC=0x7B;
DDRC=0x04;

// Port D initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=P State6=P State5=P State4=P State3=P State2=P State1=P State0=P 
PORTD=0xFF;
DDRD=0x00;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 7,200 kHz
// Mode: Normal top=FFh
// OC0A output: Disconnected
// OC0B output: Disconnected
TCCR0A=0x00;
TCCR0B=0x05;
TCNT0=0x00;
OCR0A=0x00;
OCR0B=0x00;

TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

ASSR=0x00;
TCCR2A=0x00;
TCCR2B=0x00;
TCNT2=0x00;
OCR2A=0x00;
OCR2B=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
// Interrupt on any change on pins PCINT0-7: Off
// Interrupt on any change on pins PCINT8-14: Off
// Interrupt on any change on pins PCINT16-23: Off
EICRA=0x00;
EIMSK=0x00;
PCICR=0x00;

// Timer/Counter 0 Interrupt(s) initialization
TIMSK0=0x01;
// Timer/Counter 1 Interrupt(s) initialization
TIMSK1=0x00;
// Timer/Counter 2 Interrupt(s) initialization
TIMSK2=0x00;

// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: On
// USART Transmitter: On
// USART0 Mode: Asynchronous
// USART Baud rate: 9600
UCSR0A=0x00;
UCSR0B=0x98;
UCSR0C=0x06;
UBRR0H=0x00;
UBRR0L=0x2F;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
ADCSRB=0x00;

// Global enable interrupts
#asm("sei")

i2c_init();
PORTC.2=1;

while (1)
      {
      // Place your code here
      

      };
}
